DigitalSignSingleText = {}

local MaxSingleLineChars = 12
local FontName = "GENERIC"

local FirstLineAlignment = RenderText.ALIGN_CENTER
local FirstLineSize = 0.11
local FirstLineScaleX = 1
local FirstLineScaleY = 1
local FirstLineEmissiveScale = 0.2
local FirstLineColor = {0.603, 0.507, 0, 0.9}

function DigitalSignSingleText:IsDigitalSignSingleText(text)
    if text == "Weiler" then
        return true
    end
    if text == "Weiler" then
        return true
    end
    if text == "Weiler" then
        return true
    end
    if text == "Weiler" then
        return true
    end
    if text == "Weiler" then
        return true
    end
    if text == "Weiler" then
        return true
    end
    return false
end

function DigitalSignSingleText:DistanceFrom(x1, y1, x2, y2)
    return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2)
end

function DigitalSignSingleText:GetAllSigns()
    Signs = {}
    for i = #g_currentMission.placeableSystem.placeables, 1, -1 do
        local Cplaceable = g_currentMission.placeableSystem.placeables[i]
        if Cplaceable ~= nil and DigitalSignSingleText:IsDigitalSignSingleText(Cplaceable.storeItem.name) then
            Signs[table.getn(Signs) + 1] = Cplaceable
        end
    end
    return Signs
end

function DigitalSignSingleText:GetSignInRange()
    local Px, Py, Pz = getTranslation(g_currentMission.player.rootNode)

    local AllSigns = DigitalSignSingleText:GetAllSigns()
    if AllSigns == nil or table.getn(AllSigns) < 1 then
        return
    end

    for i = 1, table.getn(AllSigns) do
        local CurrentSign = AllSigns[i]

        if
            DigitalSignSingleText:DistanceFrom(Px, Pz, CurrentSign.position.x, CurrentSign.position.z) < 1 and
                CurrentSign:getOwnerFarmId() == g_currentMission.player.farmId
         then
            return CurrentSign
        end
    end

    return nil
end

function DigitalSignSingleText:FormatText(Text)
    for i = 1, (MaxSingleLineChars - string.len(Text)) / 2 do
        Text = "_" .. Text .. "_"
    end

    return Text
end

local TimerI = 0
local MaxTimerI = g_gameSettings:getValue(GameSettings.SETTING.FRAME_LIMIT)/2

function DigitalSignSingleText:Timer()
    TimerI = TimerI + 1
    if TimerI < MaxTimerI then
        return false
    end
    TimerI = 0
    return true
end

function DigitalSignSingleText:update()
    if not DigitalSignSingleText:Timer() then
        return
    end

    local AllSigns = DigitalSignSingleText:GetAllSigns()
    if AllSigns == nil or table.getn(AllSigns) < 1 then
        return
    end

    DigitalSignSingleText:InitSings(AllSigns)
    DigitalSignSingleText:UpdateTexts(AllSigns)
end

function DigitalSignSingleText:loadMap()
    if g_currentMission:getIsClient() then
        Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, DigitalSignSingleText.RegisterActionEvents)
        Player.updateActionEvents = Utils.appendedFunction(Player.updateActionEvents, DigitalSignSingleText.UpdateActionEvents)
    end
end

function DigitalSignSingleText:RegisterActionEvents()
    if self.isClient then
        DigitalSignSingleText.events = {}
        local valid1, eventId1 =
            g_inputBinding:registerActionEvent(
            InputAction.TOWNSIGN_SINGLERENAMEACTION,
            "TOWNSIGN_SINGLERENAMEACTION",
            DigitalSignSingleText.RenameEvent,
            false,
            true,
            false,
            true
        )
        if valid1 then
            table.insert(DigitalSignSingleText.events, eventId1)
            g_inputBinding:setActionEventTextVisibility(eventId1, true)
            self.RenameEventId = eventId1
        end
    end
end

function DigitalSignSingleText:UpdateActionEvents()
    if self.RenameEventId ~= nil then
        local InRange = DigitalSignSingleText:GetSignInRange()
        g_inputBinding:setActionEventActive(self.RenameEventId, InRange ~= nil)
    end
end

function DigitalSignSingleText:ValidateText(text)

    local NewText = ""
    for i = 1, #text do
        local c = text:sub(i, i)
        if c ~= TextSeparator then
            NewText = NewText .. c
        end
    end

    return NewText
end

function DigitalSignSingleText:RenameEvent(actionName, keyStatus, arg3, arg4, arg5)
    local InRange = DigitalSignSingleText:GetSignInRange()
    if InRange == nil then
        return
    end

    g_gui:showTextInputDialog(
        {
            text = g_i18n:getText("text_WeilerRenameInGui"),
            defaultText = InRange:getName(),
            callback = function(text, yes)
                if yes == false then return end;
                InRange:setName(text, false);
            end
        }
    )
end

function DigitalSignSingleText:InitSings(AllSigns)
    for i = 1, table.getn(AllSigns) do
        local CurrentSign = AllSigns[i]

        local FontMaterial = g_materialManager:getFontMaterial(FontName, CurrentSign.customEnvironment)

        if CurrentSign.FontMaterialN == nil or CurrentSign.CharacterLineN == nil then
            local TextNode = getChildAt(getChildAt(getChildAt(CurrentSign.rootNode, 0), 0), 0)
            DgbTextNode = TextNode;
            CurrentSign.FontMaterialN = FontMaterial
            CurrentSign.CharacterLineN =
                FontMaterial:createCharacterLine(
                TextNode,
                MaxSingleLineChars,
                FirstLineSize,
                FirstLineColor,
                nil,
                FirstLineEmissiveScale,
                FirstLineScaleX,
                FirstLineScaleY,
                FirstLineAlignment
            )
        end

        if CurrentSign.FontMaterialS == nil or CurrentSign.CharacterLineS == nil then
            local TextNode = getChildAt(getChildAt(getChildAt(CurrentSign.rootNode, 0), 0), 1)
            CurrentSign.FontMaterialS = FontMaterial
            CurrentSign.CharacterLineS =
                FontMaterial:createCharacterLine(
                TextNode,
                MaxSingleLineChars,
                FirstLineSize,
                FirstLineColor,
                nil,
                FirstLineEmissiveScale,
                FirstLineScaleX,
                FirstLineScaleY,
                FirstLineAlignment
            )
        end
    end
end


function DigitalSignSingleText:UpdateTexts(AllSigns)
    for i = 1, table.getn(AllSigns) do
        local CurrentSign = AllSigns[i]

        if CurrentSign.CharacterLineN ~= nil then
            local Text = CurrentSign:getName();

            CurrentSign.FontMaterialN:updateCharacterLine(CurrentSign.CharacterLineN, DigitalSignSingleText:FormatText(Text))
            CurrentSign.FontMaterialS:updateCharacterLine(CurrentSign.CharacterLineS, DigitalSignSingleText:FormatText(Text))
        end
    end
end

addModEventListener(DigitalSignSingleText)
